#!/usr/bin/env perl
#!/bin/env perl

## Name: extractintrons.pl
## Version: 1.2.5
## Date: May 03, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# join paired-end mates
# --input, input bed12 file
# --output, output bed file
# --junction, intron list file
## by default, print spliced segments from joined reads into output
## --junction, print joined reads into output and print introns into intron list

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $junction = "intron.bed";
my $output = "output.bed";
my $line;
my @args;
my @pos;
my @newpos;
my @blocksize;
my @blockpos;
my $i;
my $exonnum;
my $exonpos;
my $exonsize;
my $strand;
my $genestart;
my $geneend;

usage() if (@ARGV < 2 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "junction=s" => \$junction,
			 "output=s" => \$output)
	    or defined $help);

if (defined $input) {
    open(STDIN, $input);
}
open(OUT, ">$output");
open(INTRON, ">$junction");

while ($line=<STDIN>) {
    @pos = ();			# all exon postions

    #< first mate
    chomp($line);
    @args = split("\t", $line);
    #>
        
    #< first mate
    @blocksize = split(",",$args[10]);
    @blockpos = split(",",$args[11]);
    for ($i=0; $i<$args[9]-1; $i++) {
	push(@pos, $args[1]+$blockpos[$i]+$blocksize[$i], $args[1]+$blockpos[$i+1])
    }
    $strand = $args[5];
    #>
    
    #< gene positions
    $genestart = $args[1];
    $geneend = $args[2];
    #>  

    @newpos = ($genestart, @pos, $geneend);
    
    #< print joined read
    $exonnum = ($#newpos+1)/2;
    if ($#newpos==1) {
	$exonsize = $geneend - $genestart;
	print OUT "$args[0]\t$genestart\t$geneend\t$args[3]\t$args[4]\t$strand\t$genestart\t$geneend\t255,0,0\t$exonnum\t$exonsize\t0\n";
    }
    else {
	print OUT "$args[0]\t$genestart\t$geneend\t$args[3]\t$args[4]\t$strand\t$genestart\t$geneend\t255,0,0\t$exonnum\t";
	for ($i=0; $i<$#newpos-1; $i=$i+2) {
	    $exonsize = $newpos[$i+1]-$newpos[$i];
	    print OUT "$exonsize,";
	    print INTRON "$args[0]\t$newpos[$i+1]\t$newpos[$i+2]\n";
	}
	$exonsize = $geneend - $newpos[$i];
	print OUT "$exonsize\t";
	for ($i=0; $i<$#newpos-1; $i=$i+2) {
	    $exonpos = $newpos[$i]-$genestart;
	    print OUT "$exonpos,";
	}
	$exonpos = $newpos[$i]-$genestart;
	print OUT "$exonpos\n";
    }
    #>
}

close(STDIN);
close(OUT);
close(INTRON);
exit 0;

sub max {
    if ($_[0] > $_[1]) {
	$_[0];
    }
    else {
	$_[1];
    }
}

sub min {
    if ($_[0] < $_[1]) {
	$_[0];
    }
    else {
	$_[1];
    }
}

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl joinpairbed12.pl --input BED12 [--junction INTRON_OUT] [--output BED_OUT] [--help|?] \n";
    exit 1;
}
