#!/usr/bin/env perl
#!/bin/env perl

## Name: gtf2gff.pl
## Version: 1.2.5
## Date: May 03, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# convert gtf to gff3
# --input, input gtf file with gene_id in note
# --output, output gff

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $output;
my $line;
my @args;
my @argsgene;
my @argstsc;
my $gene_exist;
my $tsc_exist;
my $extcodon_exist;
my $gene_start;
my $gene_end;
my $i;
my @gene;
my @exons = ();
my $current_id="";
my $current_name="";

usage() if (@ARGV < 2 or
            ! GetOptions("help|?" => \$help,
                         "input=s" => \$input,
			 "output=s" => \$output)
            or defined $help);


##-- check if gene features exist, if not then create gene features from start and stop codons
$gene_exist=`awk '{if (\$3=="gene") print \$0}' $input`;
$tsc_exist=`awk '{if (\$3=="transcript") print \$0}' $input`;

##-- check if extremity codon features exist
$extcodon_exist=`awk '{if (\$3=="start_codon" || \$3=="stop_codon" || \$3=="end_codon" || \$3=="begin_codon") print \$0}' $input`;

#if (!$gene_exist && !$tsc_exist && !$extcodon_exist) {
    #print "Error: not any of gene, transcript, start_codon, begin_codon, stop_codon nor end_codon are found.";
    #exit 1;
#}
if (!$gene_exist && !$tsc_exist) {
    system("sort -k1,1 -k9 $input > ${input}.sort");
    open(STDIN, $input.".sort");
    system("rm ${input}.sort");
} elsif (defined $input) {
    open(STDIN, $input);
}

if (defined $output) {
    open(STDOUT, ">$output");
}

while ($line = <STDIN>) {
    chomp($line);
    ##-- if not a comment
    if (substr($line, 0, 1) ne "#") {
	@args = split("\t", $line);
	@argsgene = split("gene_id", $args[-1]);
	@argstsc = split("transcript_id", $args[-1]);
	if ($#argsgene > 0) {
	    @argsgene = split(";", $argsgene[-1]);
	    $argsgene[0] = &trim($argsgene[0]);
	    if ($#argstsc > 0) {
		@argstsc = split(";", $argstsc[-1]);
		$argstsc[0] = &trim($argstsc[0]);
	    }
	    else {
		$argstsc[0] = $argsgene[0];
	    }
	}
	else {
	    if ($#argstsc > 0) {
		@argstsc = split(";", $argstsc[-1]);
		$argstsc[0] = &trim($argstsc[0]);
		$argsgene[0] = $argstsc[0];
	    }
	    else {
		print "Error: transcript_id or gene_id not found\n";
		exit 1;
	    }
	}
	
	if ($gene_exist || $tsc_exist) {
	    if (!$gene_exist && $tsc_exist && $args[2] eq "transcript") {
		$args[2] = "gene";
	    }
	    for ($i=0; $i<8; $i++) {
		print STDOUT "$args[$i]\t";
	    }
	    if ($args[2] eq "gene" || $args[2] eq "pseudogene" || $args[2] eq "transcript") {
		print STDOUT "ID=$argsgene[0];";
		print STDOUT "Name=$argstsc[0]\n";
	    }
	    else {
		print STDOUT "Parent=$argsgene[0]\n";
	    }
	}
	else {
	    if ($argstsc[0] ne $current_name && $current_name ne "") {
		for ($i=0; $i<9; $i++) {
		    $gene[$i] = $exons[$i];
		}
		$gene[2] = "gene\t";
		$gene[3] = "$gene_start\t";
		$gene[4] = "$gene_end\t";
		$gene[7] = ".\t";
		$gene[8] = "ID=$current_id;Name=$current_name\n";
		print STDOUT (@gene);
		print STDOUT (@exons);
		@exons = ();
		for ($i=0; $i<8; $i++) {
		    push(@exons, "$args[$i]\t");
		}
		@exons = (@exons, "Parent=$argsgene[0]\n");
		$gene_start = $args[3];
		$gene_end = $args[4];
		$current_id = $argsgene[0];
		$current_name = $argstsc[0];
	    }
	    else {
		if (! $current_name) {
		    $gene_start = $args[3];
		    $gene_end = $args[4];
		}
		for ($i=0; $i<8; $i++) {
		    push(@exons, "$args[$i]\t");
		}
		@exons = (@exons, "Parent=$argsgene[0]\n");
		
		if ($args[2] eq "exon") {
		    if ($args[3] < $gene_start) {
			$gene_start = $args[3];
		    }
		    if ($args[4] > $gene_end) {
			$gene_end = $args[4];
		    }
		}
		$current_id = $argsgene[0];
		$current_name = $argstsc[0];
	    }
	}
    }
}
if ($current_id) {
for ($i=0; $i<9; $i++) {
    $gene[$i] = $exons[$i];
}
$gene[2] = "gene\t";
$gene[3] = "$gene_start\t";
$gene[4] = "$gene_end\t";
$gene[7] = ".\t";
$gene[8] = "ID=$current_id;Name=$current_name\n";
print (@gene);
print (@exons);
}

close(STDIN);
close(STDOUT);
exit 0;

sub trim {
    my $str = shift;
    
    ##-- remove white spaces at the beginning and at the end
    $str =~ s/^\s+|\s+$//g;
    ##-- remove quotes at the beginning and at the end
    $str =~ s/^\"+|\"+$//g;
    
    return $str;
}

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl gtf2gff.pl --input IN [--output OUT] [--help|?] \n";
    exit 1;
}
