#!/usr/bin/env perl
#!/bin/env perl

## Name: makeCoverage.pl
## Version: 1.2.5
## Date: May 03, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Create scripts for coverage profile computation for each chromosome and for each library

use strict;
use warnings;
use Getopt::Long qw(:config no_ignore_case);

my $help;
my $cluster;
my $bam="instance.bam";
my $path=".";
my $genome="genomesize.txt";
my $node="1";
my $src;
my $memory="2G";
my $output="output";
my $librarytype="fr-unstranded";
my $wholeDE="";
my $cpu=1;

my $instance;
my $base=0;
my $line;
my $gff;
my $htseqstranded="no";
my $perlsrc;
my @args;
my @chromosomes;

usage() if (($#ARGV<0) or !(GetOptions (
				'help|?' => \$help,
				'cluster' => \$cluster,
				'wholeDE' => \$wholeDE,
				'bam=s' => \$bam,
				'path=s' => \$path,
				'node=s' => \$node,
				'src=s' => \$src,
				'library-type=s' => \$librarytype,
				'GenomeSize=s' => \$genome,
				'memory=s' => \$memory,
				'CPU=i' => \$cpu,
				'outputPrefix=s' => \$output
			    ))
	    or defined($help));

$perlsrc = $src."/perl";

$instance = &get_bam_name($bam);
if ($wholeDE) {
    if ($librarytype eq "fr-unstranded") {
	$htseqstranded = "no";
    } elsif ($librarytype eq "fr-firststrand") {
	$htseqstranded = "reverse";
    } elsif ($librarytype eq "fr-secondstrand") {
	$htseqstranded = "yes";
    }
}

open(CHR, "$genome") or die("Genome size file not found\n");
@chromosomes = ();
while ($line = <CHR>) {
    chomp($line);
    @args = split("\t", $line);
    push(@chromosomes, $args[0]);
}
close(CHR);

for (my $i=0; $i<=$#chromosomes; $i++) {
    open(OUT, ">${output}_$chromosomes[$i].sh");
    if ($cluster) {
	print OUT "#\$ -S /bin/bash\n";
	print OUT "#\$ -N runCoverage_${instance}_$chromosomes[$i]\n";
	print OUT "#\$ -V\n";
	print OUT "#\$ -cwd\n";
	print OUT "##\$ -q main.q\@node".$node."\n";
	print OUT "#\$ -pe parallel ".$cpu."\n";
	print OUT "#\$ -l h_vmem=".$memory."\n";
	print OUT "\n";
    }
    else {
	print OUT "#!/usr/bin/env bash\n";
	print OUT "#!/bin/env bash\n";
	print OUT "\n";
    }
    
    print OUT "##variables\n";
    print OUT "SRC=$perlsrc\n";
    print OUT "GFF=\$1\n";
    print OUT "DIR=$path\n\n";
    
    print OUT "##compute coverage profile for each transcript in a chromosome (long)\n";
    
    ##NEG
    print OUT "echo \"awk -v arg=$chromosomes[$i] \'\\\$1==arg\' \$DIR/rnaseq/UNS_NEG_${instance}.tsv \> \$DIR/rnaseq/count/NEG_${instance}_$chromosomes[$i].tsv\" > \$DIR/script/sense_coverage_neg_${instance}_$chromosomes[$i].sh\n";
    if ($wholeDE) {
	#print OUT "echo \"cat \$DIR/rnaseq/header.sam \> \$DIR/rnaseq/NEG_${instance}_$chromosomes[$i].sam\" >> \$DIR/script/sense_coverage_neg_${instance}_$chromosomes[$i].sh\n";
	print OUT "echo \"awk -v arg=$chromosomes[$i] \'\\\$3==arg\' \$DIR/rnaseq/UNS_NEG_refined_${instance}.sam \> \$DIR/rnaseq/NEG_${instance}_$chromosomes[$i].sam\" >> \$DIR/script/sense_coverage_neg_${instance}_$chromosomes[$i].sh\n";
    }
    print OUT "echo \"\$SRC/getbasecovchr.pl -i \$DIR/rnaseq/gff/\${GFF}NEG_noexon_$chromosomes[$i].gff -c \$DIR/rnaseq/count/NEG_${instance}_$chromosomes[$i].tsv -o \$DIR/rnaseq/NEG_${instance}_$chromosomes[$i].cur -b $base\" >> \$DIR/script/sense_coverage_neg_${instance}_$chromosomes[$i].sh\n";
    print OUT "echo \"rm \$DIR/rnaseq/count/NEG_${instance}_$chromosomes[$i].tsv\" >> \$DIR/script/sense_coverage_neg_${instance}_$chromosomes[$i].sh\n\n";

    ##POS
    print OUT "echo \"awk -v arg=$chromosomes[$i] \'\\\$1==arg\' \$DIR/rnaseq/UNS_POS_${instance}.tsv \> \$DIR/rnaseq/count/POS_${instance}_$chromosomes[$i].tsv\" > \$DIR/script/sense_coverage_pos_${instance}_$chromosomes[$i].sh\n";
    if ($wholeDE) {
	#print OUT "echo \"cat \$DIR/rnaseq/header.sam \> \$DIR/rnaseq/POS_${instance}_$chromosomes[$i].sam\" >> \$DIR/script/sense_coverage_pos_${instance}_$chromosomes[$i].sh\n";
	print OUT "echo \"awk -v arg=$chromosomes[$i] \'\\\$3==arg\' \$DIR/rnaseq/UNS_POS_refined_${instance}.sam \> \$DIR/rnaseq/POS_${instance}_$chromosomes[$i].sam\" >> \$DIR/script/sense_coverage_pos_${instance}_$chromosomes[$i].sh\n";
    }
    print OUT "echo \"\$SRC/getbasecovchr.pl -i \$DIR/rnaseq/gff/\${GFF}POS_noexon_$chromosomes[$i].gff -c \$DIR/rnaseq/count/POS_${instance}_$chromosomes[$i].tsv -o \$DIR/rnaseq/POS_${instance}_$chromosomes[$i].cur -b $base\" >> \$DIR/script/sense_coverage_pos_${instance}_$chromosomes[$i].sh\n";
    print OUT "echo \"rm \$DIR/rnaseq/count/POS_${instance}_$chromosomes[$i].tsv\" >> \$DIR/script/sense_coverage_pos_${instance}_$chromosomes[$i].sh\n\n";
    
    ##UNS
    print OUT "echo \"awk -v arg=$chromosomes[$i] \'\\\$1==arg\' \$DIR/rnaseq/UNS_${instance}.tsv \> \$DIR/rnaseq/count/UNS_${instance}_$chromosomes[$i].tsv\" > \$DIR/script/sense_coverage_uns_${instance}_$chromosomes[$i].sh\n";
    if ($wholeDE) {
	#print OUT "echo \"cat \$DIR/rnaseq/header.sam \> \$DIR/rnaseq/UNS_${instance}_$chromosomes[$i].sam\" >> \$DIR/script/sense_coverage_uns_${instance}_$chromosomes[$i].sh\n";
	print OUT "echo \"awk -v arg=$chromosomes[$i] \'\\\$3==arg\' \$DIR/rnaseq/UNS_refined_${instance}.sam \> \$DIR/rnaseq/UNS_${instance}_$chromosomes[$i].sam\" >> \$DIR/script/sense_coverage_uns_${instance}_$chromosomes[$i].sh\n";
    }

    print OUT "echo \"\$SRC/getbasecovchr.pl -i \$DIR/rnaseq/gff/\${GFF}UNS_noexon_$chromosomes[$i].gff -c \$DIR/rnaseq/count/UNS_${instance}_$chromosomes[$i].tsv -o \$DIR/rnaseq/UNS_${instance}_$chromosomes[$i].cur -b $base\" >> \$DIR/script/sense_coverage_uns_${instance}_$chromosomes[$i].sh\n";
    print OUT "echo \"rm \$DIR/rnaseq/count/UNS_${instance}_$chromosomes[$i].tsv\" >> \$DIR/script/sense_coverage_uns_${instance}_$chromosomes[$i].sh\n\n";

    print OUT "ls $path/script/sense_coverage_\*_${instance}_$chromosomes[$i].sh | xargs -P $cpu -I {} bash -c \"bash {}\"\n";
    print OUT "rm $path/script/sense_coverage_\*_${instance}_$chromosomes[$i].sh\n";

    ##concatenate NEG, POS, UNS
    print OUT "cat \$DIR/rnaseq/NEG_${instance}_$chromosomes[$i].cur \$DIR/rnaseq/POS_${instance}_$chromosomes[$i].cur \$DIR/rnaseq/UNS_${instance}_$chromosomes[$i].cur | sort -k2,2n -k3,3n > \$DIR/curve/${instance}_$chromosomes[$i].cur\n";
    print OUT "rm \$DIR/rnaseq/NEG_${instance}_$chromosomes[$i].cur \$DIR/rnaseq/POS_${instance}_$chromosomes[$i].cur \$DIR/rnaseq/UNS_${instance}_$chromosomes[$i].cur\n";
    print OUT "echo \"Done: compute coverage profile for each transcript in $chromosomes[$i]\"\n\n";

    ## htseq-count
    if ($wholeDE) {
	print OUT "##count gene coverage by htseq-count (long)\n";
	print OUT "if [ -s \"\$DIR/rnaseq/gff/\${GFF}NEG_noexon_$chromosomes[$i].gff\" ] && [ -s \"\$DIR/rnaseq/NEG_${instance}_$chromosomes[$i].sam\" ]\n";
	print OUT "then\n";
	print OUT "htseq-count -i ID --type=gene --stranded=$htseqstranded --mode=intersection-strict \$DIR/rnaseq/NEG_${instance}_$chromosomes[$i].sam \$DIR/rnaseq/gff/\${GFF}NEG_noexon_$chromosomes[$i].gff > \$DIR/rnaseq/htseqcount/NEG_${instance}_$chromosomes[$i].cnt\n";
	print OUT "head -n -5 \$DIR/rnaseq/htseqcount/NEG_${instance}_$chromosomes[$i].cnt > \$DIR/rnaseq/htseqcount/${instance}_$chromosomes[$i].cnt\n";
	print OUT "rm \$DIR/rnaseq/htseqcount/NEG_${instance}_$chromosomes[$i].cnt\n";
	print OUT "fi\n";
        
	print OUT "if [ -s \"\$DIR/rnaseq/gff/\${GFF}POS_noexon_$chromosomes[$i].gff\" ] && [ -s \"\$DIR/rnaseq/POS_${instance}_$chromosomes[$i].sam\" ]\n";
	print OUT "then\n";
	print OUT "htseq-count -i ID --type=gene --stranded=$htseqstranded --mode=intersection-strict \$DIR/rnaseq/POS_${instance}_$chromosomes[$i].sam \$DIR/rnaseq/gff/\${GFF}POS_noexon_$chromosomes[$i].gff > \$DIR/rnaseq/htseqcount/POS_${instance}_$chromosomes[$i].cnt\n";
	print OUT "head -n -5 \$DIR/rnaseq/htseqcount/POS_${instance}_$chromosomes[$i].cnt >> \$DIR/rnaseq/htseqcount/${instance}_$chromosomes[$i].cnt\n";
	print OUT "rm \$DIR/rnaseq/htseqcount/POS_${instance}_$chromosomes[$i].cnt\n";
	print OUT "fi\n";
	
	print OUT "if [ -s \"\$DIR/rnaseq/gff/\${GFF}UNS_noexon_$chromosomes[$i].gff\" ] && [ -s \"\$DIR/rnaseq/UNS_${instance}_$chromosomes[$i].sam\" ]\n";
	print OUT "then\n";
	print OUT "htseq-count -i ID --type=gene --stranded=$htseqstranded --mode=intersection-strict \$DIR/rnaseq/UNS_${instance}_$chromosomes[$i].sam \$DIR/rnaseq/gff/\${GFF}UNS_noexon_$chromosomes[$i].gff > \$DIR/rnaseq/htseqcount/UNS_${instance}_$chromosomes[$i].cnt\n";
	print OUT "head -n -5 \$DIR/rnaseq/htseqcount/UNS_${instance}_$chromosomes[$i].cnt >> \$DIR/rnaseq/htseqcount/${instance}_$chromosomes[$i].cnt\n";
	print OUT "rm \$DIR/rnaseq/htseqcount/UNS_${instance}_$chromosomes[$i].cnt\n";
	print OUT "fi\n";
	
	print OUT "rm \$DIR/rnaseq/NEG_${instance}_$chromosomes[$i].sam \$DIR/rnaseq/POS_${instance}_$chromosomes[$i].sam \$DIR/rnaseq/UNS_${instance}_$chromosomes[$i].sam\n";
	print OUT "echo \"Done: count gene coverage by htseq-count in $chromosomes[$i]\"\n\n";
    }
    close(OUT);
}

exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: see configure.sh \n";
    exit 1;
}

## remove extension bam or accepted_hits.bam
sub get_bam_name {
    my $bam = $_[0];
    my $extension;
    my $lastslash = rindex($bam, "/");
    
    $bam = substr($bam, $lastslash+1);
    $extension = substr($bam, -4);
    if ($extension eq ".bam") {
	$bam = substr($bam, 0, -4);
	$extension = substr($bam, -13);
	if ($extension eq "accepted_hits") {
	    $bam = substr($bam, 0, -13);
	    $extension = substr($bam, -1);
	    if ($extension eq "." || $extension eq "_") {
		$bam = substr($bam, 0, -1);
	    }
	}
    }
    else {
	print "\tError: bam file is required\n";
	exit 1;
    }
    
    return $bam;
}
