#!/usr/bin/env perl
#!/bin/env perl

## Name: segment.pl
## Version: 1.2.5
## Date: May 03, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# find segmentation from 'genomecoveragebed' ouput
# --input, input file in genomecoveragebed output format
# --output, output file
# --min, minium number of reads to consider transcription

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $output = "output.tsv";
my $minread = 1;

my $line;
my @args;
my @argsold;
my $i;

usage() if ($#ARGV < 0 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "min=i" => \$minread,
			 "output=s" => \$output)
	    or defined $help);

open(IN, $input) or die("Input file not found\n");
open(OUT, ">$output");


while ($line = <IN>) {
    chomp($line);
    @argsold = split("\t", $line);
    if ($argsold[3] >= $minread) {
	last;
    }
}

while ($line = <IN>) {
    chomp($line);
    @args = split("\t", $line);
    
    while ($args[1] <= $argsold[2] && $args[0] eq $argsold[0] && $args[3] >= $minread) {
	@argsold = fusion(\@argsold, \@args);
	$line = <IN>;
	if ($line) {
	    chomp($line);
	    @args = split("\t", $line);
	}
	else {
	    @args = (0,0,0,0);
	}
    }
    if ($argsold[3] >= $minread) {
	for ($i=0; $i<$#argsold; $i++) {
	    print OUT "$argsold[$i]\t";
	}
	print OUT "$argsold[$i]\n";
    }

    @argsold = @args;
    while ($line && $argsold[3]<$minread) {
	$line = <IN>;
	if ($line) {
	    chomp($line);
	    @argsold = split("\t", $line);
	}
    }
    if ($line && eof) {
	if ($argsold[3] >= $minread) {
	    for ($i=0; $i<$#argsold; $i++) {
		print OUT "$argsold[$i]\t";
	    }
	    print OUT "$argsold[$i]\n";
	}
    }
}
close(IN);
close(OUT);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl segment.pl --input IN [--min MINREAD] [--output OUT] [--help|?]\n";
    exit 1;
}

sub max {
    if ($_[0] > $_[1]) {
	$_[0];
    }
    else {
	$_[1];
    }
}

sub min {
    if ($_[0] < $_[1]) {
	$_[0];
    }
    else {
	$_[1];
    }
}


sub fusion {
    my ($a, $b) = @_;
    my @ref = @{$a};
    $ref[1] = min(${$a}[1], ${$b}[1]);	
    $ref[2] = max(${$a}[2], ${$b}[2]);
    $ref[3] = max(${$a}[3], ${$b}[3]);
    return @ref;
}
