#!/usr/bin/env perl
#!/bin/env perl

## Name: tsccomparechr.pl
## Version: 1.2.5
## Date: May 03, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

## discover new transcripts and extend existing transcripts while comparing a new gff to a reference gff
# --input, input gff file
# --reference, reference gff file
# --unstranded, while new gff is not directional
# --extend, limit to be considered an extended transcript, in case 'unstranded'
# --antisense, output antisense transcript file, in case 'unstranded'
# --nc, new unannotated transcript output file
# --output, output transcript file obtained from RNA-seq data
# --pseudointron, taking into account introns in extended regions

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $nc = "nc.gff";
my $reference;
my $unstranded;
my $pseudointron;
my $extend = 200;
my $output = "output.gff";
my $antisense = "antisense.gff";
my $linenew;
my $lineref;
my @argsref;
my @argsnew;
my @argsprev;
my @argsnewprev;
my $start = -1;
my $end = -1;
my $endprev;
my $i;
my @noteprev;
my @noteref;
my @notenew;
my @intronsref;
my @intronsnew;
my $tmp;

$argsnew[4] = -1;
$argsnewprev[4] = -1;

usage() if (@ARGV < 4 or
            ! GetOptions("help|?" => \$help,
                         "unstranded" => \$unstranded,
			 "pseudointron" => \$pseudointron,
			 "input=s" => \$input,
			 "reference=s" => \$reference,
			 "extend=i" => \$extend,
			 "antisense=s" => \$antisense,
			 "output=s" => \$output,
			 "nc=s" => \$nc)
            or defined $help);

open(IN, $input) or die("Input file not found\n");
open(REF, $reference) or die("Reference file not found\n");
open(NC, ">$nc");
open(OUT, ">$output");
if ($unstranded) {
    open(OVL, ">$antisense");
}

 WHOLE: {
     while ($lineref = <REF>) {
	 chomp($lineref);
	 @argsref = split("\t", $lineref);
	 ##print "REF: $lineref\n";
	 
	 if ($argsnewprev[4] > $end) {
	     $end = $argsnewprev[4];
	 }
	 
	 #< the previous transcript should end before the start of the current reference if these two are not overlapped
	 if ($end >= $argsref[3]) {
	     $end = max($argsref[3]-1, $argsprev[4]);
	 }
	 #>
	 
	 #< the end of the previous transcript
	 $endprev = $argsprev[4];
	 #>

	 #> print the previous transcript, ifnot before the first transcript in the reference file
	 if ($end > $start) {
	     $argsprev[3] = $start;
	     $argsprev[4] = $end;
	     unshift(@intronsref, $start);
	     push(@intronsref, $end);
	     for ($i=0; $i<$#argsprev; $i++) {
		 print OUT "$argsprev[$i]\t";
	     }
	     @noteprev = split("exons=", $argsprev[$i]);
	     if ($pseudointron) {
		 print OUT "$noteprev[0]exons=@intronsref\n";
	     }
	     else {
		 print OUT "$argsprev[$i]\n";
	     }
	 }
	 #>
	 
	 #< take all new transcripts coming before the current reference 
	 while ($argsnew[4] < $argsref[3]-1) {
	     if ($argsnew[4] > -1 && $linenew) {
		 @noteref = split(";", $argsref[8]);
		 @notenew = split("exons=", $linenew);
		 print NC "$notenew[0] ";
		 if ($end > $start) {
		     @noteprev = split(";", $argsprev[8]);
		     print NC "left=$noteprev[0]; right=$noteref[0]\n";
		 }
		 else {
		     print NC "right=$noteref[0]\n";
		 }
		 print OUT "$linenew\n";
	     }
	     $linenew = <IN>;
	     if ($linenew) {
		 chomp($linenew);
		 @argsnew = split("\t", $linenew);
	     }
	     else {
		 last;
	     }
	 }
	 #>
	 
	 if ($end == $argsref[3]-1) {
	     $start = $endprev+1;
	 }
	 else {
	     $start = $argsref[3];
	 }
	 $end = $argsref[4];
	 
	 @intronsref = split("exons=", $argsref[8]);
	 @intronsref = split(" ", $intronsref[1]);
	 shift(@intronsref);
	 pop(@intronsref);
	 #< throw all new transcripts overlapping with/including the current reference
	 while ($linenew && $argsnew[3] <= $argsref[4]+1) {
	     if ($unstranded) {
		 #< in case non-directional, get shortly overlapped transcripts on the left or right
		 if (($extend <= $argsref[4]-$argsnew[4] && 
		      $extend <= $argsref[3]-$argsnew[3] &&
		      $argsnew[4]-$argsref[3] <= $argsref[3]-$argsnew[3] && 
		      $argsnew[4]-$argsref[3] <= $argsref[4]-$argsnew[4]) || 
		     ($extend <= $argsnew[4]-$argsref[4] && 
		      $extend <= $argsnew[3]-$argsref[3] && 
		      $argsref[4]-$argsnew[3] <= $argsnew[4]-$argsref[4] && 
		      $argsref[4]-$argsnew[3] <= $argsnew[3]-$argsref[3])) {
		     @notenew = split("exons=", $linenew);
		     @noteref = split(";", $argsref[8]);
		     print OVL "$notenew[0] overlap=$noteref[0]\n";
		 }
	     }
	     
	     if ($argsnew[3] < $start) {
		     $start = $argsnew[3];
	     }
	     if ($argsnew[4] > $end) {
		 $end = $argsnew[4];
	     }
	     #< if new transcript starts before current reference
	     if ($argsnew[3] <= $argsref[3]) {
		 #< get introns
		 @intronsnew = split("exons=", $argsnew[8]);
		 @intronsnew = split(" ", $intronsnew[1]);
		 shift(@intronsnew);
		 pop(@intronsnew);
		 #>
		 
		 #< remove introns overlapped with current reference
		 while ($#intronsnew >= 0 && $intronsnew[$#intronsnew] > $argsref[3]) {
		     pop(@intronsnew);
		     pop(@intronsnew);
		 }
		 #>
		 
		 #< add new introns
		 while ($#intronsnew >= 0) {
		     unshift(@intronsref, pop(@intronsnew));
		     unshift(@intronsref, pop(@intronsnew));
		 }
		 #>
	     }
	     #>
	     #< if new transcript ends after current reference
	     if ($argsnew[4] >= $argsref[4]) {
		 #< get introns
		 @intronsnew = split("exons=", $argsnew[8]);
		 @intronsnew = split(" ", $intronsnew[1]);
		 shift(@intronsnew);
		 pop(@intronsnew);
		 #>
		 
		 #< remove introns overlapped with current reference
		 while ($#intronsnew >= 0 && $intronsnew[0] < $argsref[4]) {
		     shift(@intronsnew);
		     shift(@intronsnew);
		 }
		 #>
		 
		 #< add new introns
		 while ($#intronsnew >= 0) {
		     push(@intronsref, shift(@intronsnew));
		     push(@intronsref, shift(@intronsnew));
		 }
		 #>
	     }
	     #>
	     @argsnewprev = @argsnew;
	     $linenew = <IN>;
	     if ($linenew) {
		 chomp($linenew);
		 @argsnew = split("\t", $linenew);
	     }
	     else {
		 last;
	     }
	 }
	 #>
	 @argsprev = @argsref;
     }
}

#< print the last reference
if ($#argsprev >= 0) {
    $argsprev[3] = $start;
    $argsprev[4] = $end;
    unshift(@intronsref, $start);
    push(@intronsref, $end);
    for ($i=0; $i<$#argsprev; $i++) {
	print OUT "$argsprev[$i]\t";
    }
    @noteprev = split("exons=", $argsprev[$i]);
    if ($pseudointron) {
	print OUT "$noteprev[0]exons=@intronsref\n";
    }
    else {
	print OUT "$argsprev[$i]\n";
    }
}
#>

#< transcripts coming after the last reference
do {
    if ($linenew) {
	@notenew = split("exons=", $linenew);
	if ($#argsprev >= 0) {
	    @noteprev = split(";", $argsprev[8]);
	    print NC "$notenew[0] left=$noteprev[0]\n";
	}
	else {
	    print NC "$notenew[0]\n";
	}
	print OUT "$linenew\n";
    }
    $linenew = <IN>;
    if ($linenew) {
	chomp($linenew);
    }
} until (!$linenew && eof(IN));
#>

close(REF);
close(IN);
close(NC);
close(OVL);
close(OUT);
exit 0;

sub max {
    if ($_[0] > $_[1]) {
        $_[0];
    }
    else {
        $_[1];
    }
}

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl tsccomparechr.pl --input GFF --reference REF [--pseudointron] [--unstranded] [--extend EXT] [--nc NC] [--antisense ANTI] [--output OUT] [--help|?] \n";
    exit 1;
}
